'From Smalltalk/X, Version:5.2.8 on 16-01-2008 at 01:34:53'                     !

"{ Package: 'SpreadSheet:all' }"

Object subclass:#ArrayedTableRow
	instanceVariableNames:'cells'
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

!ArrayedTableRow methodsFor:'accessing'!

cellAt:cellIndex 
        "gets cell without resizing anything"

        (cellIndex > (self cells size)) ifTrue: [
                "row too short, return nil"
                ^ nil.
        ] ifFalse: [
                "row long enough, return its value"
                ^ self cells at: cellIndex.
        ].
!

cellAt:cellIndex put:anObject 
    "puts anObject at given index"
    
    self resizeToIndex:cellIndex.
    self cells at:cellIndex put:anObject
!

emptyCellAt:cellIndex 
    "empties cell at colIndex address"
    
    (cellIndex > (self cells size)) ifFalse:[
        "row long enough, empty cell"
        self cells at:cellIndex put:nil.
    ].
! !

!ArrayedTableRow methodsFor:'converting'!

asArray
  ^ self cells asArray.
! !

!ArrayedTableRow methodsFor:'modifying'!

delete:number cellsAt:cellIndex 
    "Delete number of cells at position given by cellIndex. Moves remaining cells to the left"
    
    (cellIndex <= (self cells size)) ifTrue:[
        |maxIndex|

        maxIndex := (cellIndex + number - 1) min:(self cells size).
        self cells removeFromIndex:cellIndex toIndex:maxIndex.
    ]
!

insert:number emptyCellsAt:cellIndex 
    "Insert number of cells at position given by cellIndex. Moves remaining cells to the right"
    
    (cellIndex <= (self cells size)) ifTrue:[
        self cells addAll:(Array new:number) beforeIndex:cellIndex.
    ].
!

insertCell:anObject at:cellIndex 
    "Insert number of cells at position given by cellIndex. Moves remaining cells to the right"
    
    self insert:1 emptyCellsAt:cellIndex.
    self cellAt:cellIndex put:anObject.
! !

!ArrayedTableRow methodsFor:'private'!

cells
   "gets cells OrderedColletion"
   cells ifNil: [          
     cells := OrderedCollection new: 0.
   ].

   ^ cells
!

resizeToIndex:cellIndex 
    "ensure, that collection has size at least as cellIndex"
    
    self cells changeCapacityTo:cellIndex.
    [
        cellIndex > (self cells size)
    ] whileTrue:[ self cells add:nil. ].
! !

