'From Smalltalk/X, Version:5.2.8 on 16-01-2008 at 01:34:52'                     !

"{ Package: 'SpreadSheet:all' }"

TestCase subclass:#ArrayedTableTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

!ArrayedTableTests methodsFor:'test'!

testAcessing01
    "single cell accessing"
    |tmp|

    tmp := ArrayedTable new.

     "getting non existing cell"
    self assert:(tmp cellAtRow:1 column:1) isNil.
    self assert:(tmp cellAtRow:10 column:10) isNil.

     "inserting value"
    tmp 
        cellAtRow:2
        column:3
        put:14.

     "checking"
    self assert:((tmp cellAtRow:2 column:3) = 14).
     "checking new size"
    self assert:((tmp columnsCount) = 3).
    self assert:((tmp rowsCount) = 2).

     "getting non defined cell"
    self assert:(tmp cellAtRow:1 column:1) isNil.


    tmp 
        cellAtRow:3
        column:4
        put:999.

    "checking"
    self assert:((tmp cellAtRow:3 column:4) = 999).
    self assert:((tmp columnsCount) = 4).
    self assert:((tmp rowsCount) = 3).
!

testAcessing02
    "accessing whole rows/ columns"
    |tmp|

    tmp := ArrayedTable new.

     "inserting value"
    tmp 
        cellAtRow:2
        column:3
        put:14.

     "checking new size"
    self assert:((tmp columnsCount) = 3).
    self assert:((tmp rowsCount) = 2).

    tmp 
        cellAtRow:3
        column:4
        put:999.

    self assert:((tmp columnsCount) = 4).
    self assert:((tmp rowsCount) = 3).

    self assert:((tmp rowAt:1) = #( nil nil nil nil )).
    self assert:((tmp rowAt:2) = #( nil nil 14 nil )).
    self assert:((tmp rowAt:3) = #( nil nil nil 999 )).
    self assert:((tmp columnAt:3) = #( nil 14 nil )).

    tmp columnAt:4 put:#( 1 2 ).

    "shorter definition, last row cell wil be deleted"
    self assert:((tmp columnAt:4) = #( 1 2 nil )).

     "non existing one"
    self assert:((tmp columnAt:5) = #( nil nil nil )).

    " causes extending of table"
    tmp rowAt:3 put:#( 1 2 3 4 5 ).

    self assert: ((tmp columnsCount) = 5).
    self assert: ((tmp rowsCount) = 3).

    self assert:((tmp rowAt:1) = #( nil nil nil 1 nil )).
    self assert:((tmp rowAt:2) = #( nil nil 14 2 nil )).
    self assert:((tmp rowAt:3) = #( 1 2 3 4 5 ) ).
    "not existing"
    self assert:((tmp rowAt:4) = #( nil nil nil nil nil) ).
    self assert:((tmp columnAt: 6) = #(nil nil nil) ).

    self assert: ((tmp columnAt: 5) = #(nil nil 5) ).
    self assert: ((tmp columnAt: 3) = #(nil 14 3) ).
!

testAcessing03
    "accessing whole rows/ columns + emptying"
    |tmp|

    tmp := ArrayedTable new.

    tmp rowAt: 2 put: #(11 12 13 14).
    tmp rowAt: 1 put: #(1 2 3 4).

    tmp emptyColumnAt: 3.

    self assert: ((tmp rowAt:1) = #(1 2 nil 4)).
    self assert: ((tmp rowAt:2) = #(11 12 nil 14)).

    tmp := ArrayedTable new.

    tmp columnAt: 2 put: #(11 12 13 14).
    tmp columnAt: 1 put: #(1 2 3 4).

    tmp emptyRowAt: 3.

    self assert: ((tmp columnAt:1) = #(1 2 nil 4)).
    self assert: ((tmp columnAt:2) = #(11 12 nil 14)).
!

testInserting
    "test inserting of rows, cols, etc."
    |tmp|

    "inserting rows"
    tmp := ArrayedTable new.
    tmp rowAt: 1 put: #(1 2 3 4 5).
    tmp rowAt:2 put: #(11 12 13 14 15).

    tmp insertRow: #(55 56 57) at: 2.
    self assert: ((tmp columnAt:2) = #(2 56 12)).

    tmp insertRow: #(0 0 0) at: 5.
    self assert: ((tmp columnAt: 5) = #(5 nil 15 nil nil)).
    self assert: ((tmp columnAt: 3) = #(3 57 13 nil 0)).



"inserting columns"
    tmp := ArrayedTable new.
    tmp rowAt:1 put: #(1 2 3).
    tmp rowAt:2 put: #(11 12 13).
    tmp rowAt:3 put: #(21 22 23).

    tmp insertColumn: #(1 2) at:3.
    self assert: ((tmp rowAt:1) = #(1 2 1 3 )).
    self assert: ((tmp rowAt:2) = #(11 12 2 13)).
    self assert: ((tmp rowAt:3) = #(21 22 nil 23)).

    tmp insertColumn: #(99 98) at: 6.
    self assert: ((tmp rowAt: 1) = #(1 2 1 3 nil 99)).
    self assert: ((tmp rowAt: 2) = #(11 12 2 13 nil 98)).
    self assert: ((tmp rowAt: 3) = #(21 22 nil 23 nil nil)).

    tmp insertColumn: #(4 4 4 44) at: 2.
    self assert: ((tmp rowAt: 4) = #(nil 44 nil nil nil nil nil)).
!

testInsertingEmpty
    "test inserting of empty rows, cols, etc."
    |tmp|

    "inserting rows"
    tmp := ArrayedTable new.
    tmp rowAt:1 put: #(1 2 3 4 5).
    tmp rowAt:2 put: #(11 12 13 14 15).
    tmp rowAt:3 put: #(21 22 23 24 25).

    tmp insert: 2 emptyRowsAt: 2.
    self assert: ((tmp columnAt:2) = #(2 nil nil 12 22)).

    tmp insert: 1 emptyRowsAt: 5.
    self assert: ((tmp columnAt: 3) = #(3 nil nil 13 nil 23)).


    tmp insert: 1 emptyRowsAt: 10.
    "nothing's changed, inserting empty row out of bounds"
    self assert: ((tmp columnAt: 3) = #(3 nil nil 13 nil 23)).


"inserting columns"
    tmp := ArrayedTable new.
    tmp rowAt:1 put: #(1 2 3 4 5).
    tmp rowAt:2 put: #(11 12 13 14 15).
    tmp rowAt:3 put: #(21 22 23 24 25).

    tmp insert: 2 emptyColumnsAt: 2.
    self assert: ((tmp rowAt:2) = #(11 nil nil 12 13 14 15)).

    tmp insert: 1 emptyColumnsAt: 5.
    self assert: ((tmp rowAt: 3) = #(21 nil nil 22 nil 23 24 25)).


    tmp insert: 1 emptyColumnsAt: 10.
    "nothing's changed, inserting empty col out of bounds"
    self assert: ((tmp rowAt: 3) = #(21 nil nil 22 nil 23 24 25)).
!

testIterating
    "test iterating through rows, columns and all cells"
    |tmp str|

    tmp := ArrayedTable new.
    tmp rowAt:1 put: #(1 2 3 4 5).
    tmp rowAt:2 put: #(11 12 13 14 15).

    "iterating all cells"
    str:=''.
    tmp do: [:x|
        str := str, x asString.
    ].
    self assert: (str = '123451112131415').

    "iterating all rows"
    str:=''.
    tmp rowsDo: [:x|
        x do: [:y| str := str, y asString.].
    ].
    self assert: (str = '123451112131415').

    "iterating all cols"
    str:=''.
    tmp columnsDo: [:x|
        x do: [:y| str := str, y asString.].
    ].
    self assert: (str = '111212313414515').
!

testRemoving
    "test removing of rows, cols, etc."
    |tmp|

    "removing rows"
    tmp := ArrayedTable new.
    tmp rowAt:1 put: #(1 2 3 4 5).
    tmp rowAt:2 put: #(11 12 13 14 15).
    tmp rowAt:3 put: #(21 22 23 24 25).
    tmp rowAt:4 put: #(31 32 33 34 35).
    tmp rowAt:5 put: #(41 42 43 44 45).
    tmp rowAt:6 put: #(51 52 53 54 55).

    tmp delete: 3 rowsAt: 3.
    self assert: ((tmp columnAt:2) = #(2 12 52)).

    tmp delete: 3 rowsAt: 9.
    self assert: ((tmp columnAt:2) = #(2 12 52)).
        
    tmp delete: 1 rowsAt: 1.
    self assert: ((tmp columnAt: 3) = #(13 53)).

    tmp delete: 4 rowsAt: 2.
    self assert: ((tmp columnAt: 3) = #(13)).


    "removing columns"
    tmp := ArrayedTable new.
    tmp rowAt:1 put: #(1 2 3 4 5).
    tmp rowAt:2 put: #(11 12 13 14 15).
    tmp rowAt:3 put: #(21 22 23 24 25).
    tmp rowAt:4 put: #(31 32 33 34 35).
    tmp rowAt:5 put: #(41 42 43 44 45).
    tmp rowAt:6 put: #(51 52 53 54 55).

    tmp delete: 2 columnsAt: 2.
    self assert: ((tmp rowAt:2) = #(11 14 15)).

    tmp delete: 1 columnsAt: 1.
    self assert: ((tmp rowAt: 3) = #(24 25)).

    tmp delete: 4 columnsAt: 2.
    self assert: ((tmp rowAt: 3) = #(24)).
! !

