'From Smalltalk/X, Version:5.2.8 on 16-01-2008 at 01:34:52'                     !

"{ Package: 'SpreadSheet:all' }"

CellReference subclass:#CellReferenceRange
	instanceVariableNames:'row2 col2'
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

!CellReferenceRange class methodsFor:'instance creation'!

newFromR: row1 C: col1 toR: row2 C: col2 atSheet: aSheet
  |tmp|
  tmp := self new.
  tmp setReferenceFromR: row1 C: col1 toR: row2 C: col2 atSheet: aSheet.
  ^tmp.
! !

!CellReferenceRange methodsFor:'accessing'!

asString
        " e.g.: '<sheet name>'.C1:D3 "
        ^ '''', sheet name, '''.', (col asAlphabet),(row asString),':',(col2 asAlphabet),(row2 asString).
!

value
        "returns first object within the range"

        "shouldn't it be deleted ???"
        self checkRecursion.
        (((((sheet isNil) or:[ row isNil ]) or:[ row2 isNil ]) or:[ col isNil ]) 
        or:[ col2 isNil ]) 
            ifTrue:[
                "nil"
                ^ CellUndefined new.
            ]
            ifFalse:[
                ^ sheet cellAtRow:row column:col.
            ].
! !

!CellReferenceRange methodsFor:'iteration'!

do:aBlock 
    "iterates through all cells within range"
    self checkRecursion.

    (((((sheet isNil) or:[ row isNil ]) or:[ row2 isNil ]) or:[ col isNil ]) 
        or:[ col2 isNil ]) 
            ifTrue:[
                "nil"
                aBlock value:nil.
            ]
            ifFalse:[
                row to:row2 do:[:r | 
                    col to:col2 do:[:c | 
                        aBlock value:(sheet cellAtRow:r column:c).
                    ].
                ].
            ].
!

rowsDo:aBlock 
    "iterates through all rows within range passing each row as an Array"
    
    |array|
    self checkRecursion.
    (((((sheet isNil) or:[ row isNil ]) or:[ row2 isNil ]) or:[ col isNil ]) 
        or:[ col2 isNil ]) 
            ifTrue:[
                "nil"
                aBlock value:nil.
            ]
            ifFalse:[
                row to:row2 do:[:r | 
                    array := Array new:(col2 - col + 1).
                    col to:col2 do:[:c | 
                        array at:(c - col + 1) put:(sheet cellAtRow:r column:c).
                    ].
                    aBlock value:array.
                ].
            ].
! !

!CellReferenceRange methodsFor:'listening'!

notifyColumnsFrom:colIndex reindexedBy:aNumber 
    "receives message about reindexing columns in spreadsheet"
    super notifyColumnsFrom:colIndex reindexedBy:aNumber.
    (colIndex <= col2) ifTrue: [col2 := (col2 + aNumber)].
!

notifyRowsFrom:rowIndex reindexedBy:aNumber 
    "receives message about reindexing rows in spreadsheet"
    super notifyRowsFrom:rowIndex reindexedBy:aNumber. 
    (rowIndex <= row2) ifTrue: [row2 := (row2 + aNumber)].
! !

!CellReferenceRange methodsFor:'modifying'!

setReferenceFromR:rowA C:colA toR:rowB C:colB atSheet:aSheet 
    "sets reference to any range within specified sheet"
    
    ((((aSheet isNil not and:[ rowA isNumber ]) and:[ colA isNumber ]) 
        and:[ rowB isNumber ]) and:[colB isNumber]) ifTrue:[
        "only do when good params are set"
        (sheet == aSheet) ifFalse:[
            "if there was different sheet before"
            sheet ifNotNil:[ sheet unregisterReindexListener:self ].
            aSheet registerReindexListener:self.
        ].
        sheet := aSheet.
        row := rowA min: rowB.
        row2 := rowA max: rowB.
        col := colA min: colB.
        col2 := colA max: colB.
    ].
!

setReferenceRow:rowIndex column:colIndex atSheet:aSheet 
    "sets current reference disabled in subclass"

    ^ self error: 'Does not allowed to call in this subclass, not enough parameters'.
! !

!CellReferenceRange methodsFor:'private'!

cellsDo:aBlock 
    "iterates through all cells within range without recursion detection"

    (((((sheet isNil) or:[ row isNil ]) or:[ row2 isNil ]) or:[ col isNil ]) 
        or:[ col2 isNil ]) 
            ifTrue:[
                "nil"
                aBlock value:nil.
            ]
            ifFalse:[
                row to:row2 do:[:r | 
                    col to:col2 do:[:c | 
                        aBlock value:(sheet cellAtRow:r column:c).
                    ].
                ].
            ].
! !

!CellReferenceRange methodsFor:'recursion-detection'!

checkRecursion
    "sends message deeper"
    self cellsDo:[:x |
        x checkRecursionAgainst:self.
    ].
!

checkRecursionAgainst:anObject 
    "sends message deeper"

   (anObject == self) ifTrue: [^ RecursiveDefinitionError raise.]. 
    self cellsDo:[:x | 
        x checkRecursionAgainst:anObject.
    ].
! !

!CellReferenceRange methodsFor:'spreadsheet'!

isCellReferenceRange
        ^ true.
! !

