'From Smalltalk/X, Version:5.2.8 on 16-01-2008 at 01:34:53'                     !

"{ Package: 'SpreadSheet:all' }"

TestCase subclass:#SpreadSheetExporterCSVTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

!SpreadSheetExporterCSVTests methodsFor:'private'!

getTmpFileName
 | fNameBase fName i |
 fNameBase := 'export_test_',((Timestamp now) printStringFormat: '%h%m%s%i'),'_'.
 fName:= fNameBase, (i asString).
 i := 0. 
 [ 
  i:=i+1.
  fName := fNameBase, (i asString).
  (fName asFilename) exists. 
 ] whileTrue.

 ^(fName asString)
! !

!SpreadSheetExporterCSVTests methodsFor:'test'!

test01
     |testFileName tmpSpreadSheet tmpCVSExporter tmpFile|
     testFileName:=self getTmpFileName.

     tmpSpreadSheet:= SpreadSheet new.
     tmpCVSExporter:= SpreadSheetExporterCSV new.
          tmpCVSExporter setFile: testFileName.
     tmpSpreadSheet appendRow: #(100 200 300 400 500 600).
     tmpSpreadSheet appendRow: #(700 800 '"ahoj"' 100 110 120).
     tmpSpreadSheet appendRow: #(130 140 150 160 170).
     tmpCVSExporter export: tmpSpreadSheet.

     tmpFile:= FileStream new.
     tmpFile open: testFileName withMode: 'r'.

     self assert: ((tmpFile nextLine) = '100,200,300,400,500,600').
     self assert: ((tmpFile nextLine) = '700,800,"""ahoj""",100,110,120').
     self assert: ((tmpFile nextLine) = '130,140,150,160,170,').
     self assert: ((tmpFile nextLine) = nil).

     tmpFile close.
     testFileName asFilename delete.
!

test02
     |testFileName tmpSpreadSheet tmpCVSExporter tmpFile a|
     testFileName:=self getTmpFileName.

     tmpSpreadSheet:= SpreadSheet new.
     tmpCVSExporter:= SpreadSheetExporterCSV new.
     tmpCVSExporter setFile: testFileName.
     a:= Array withSize: 3.
     a at: 1 put: (Time hour: 11 minute: 54 second: 10).
     a at: 3 put: (Date day: 13 month: 1 year: 2008).
     tmpSpreadSheet appendRow: #(100 200 300 400 500 600).
     tmpSpreadSheet appendRow: #(700 800 '"ahoj"' 100 110 120).
     tmpSpreadSheet appendRow: #(130 140 150 160 170).
     tmpSpreadSheet appendRow: a.
     tmpCVSExporter setDateFormat: '"datum: "%d. %M. %y' language: nil.
     tmpCVSExporter setTimeFormat: '"cas: "%h:%m'.
     tmpCVSExporter export: tmpSpreadSheet.

     tmpFile:= FileStream new.
     tmpFile open: testFileName withMode: 'r'.

     self assert: ((tmpFile nextLine) = '100,200,300,400,500,600').
     self assert: ((tmpFile nextLine) = '700,800,"""ahoj""",100,110,120').
     self assert: ((tmpFile nextLine) = '130,140,150,160,170,').
     self assert: ((tmpFile nextLine) = '"""cas: ""11:54",,"""datum: ""13. 1. 2008",,,').
     self assert: ((tmpFile nextLine) = nil).

     tmpFile close.
     testFileName asFilename delete.
!

test03
     |testFileName tmpSpreadSheet tmpCVSExporter tmpFile a tree branch|
     testFileName:=self getTmpFileName.

     branch := ExprBinTimes with: 2 with: 2.
     tree := ExprBinPlus with: 3 with: branch.

     tmpSpreadSheet:= SpreadSheet new.
     tmpCVSExporter:= SpreadSheetExporterCSV new.
     tmpCVSExporter setFile: testFileName.
     a:= Array withSize: 3.
     a at: 1 put: (Time hour: 11 minute: 54 second: 10).
     a at: 2 put: ((CellExpression new) root: tree).
     a at: 3 put: (Date day: 13 month: 1 year: 2008).
     tmpSpreadSheet appendRow: #(100 200 300 400 500 600).
     tmpSpreadSheet appendRow: #(700 800 '"ahoj"' 100 110 120).
     tmpSpreadSheet appendRow: #(130 140 150 160 170).
     tmpSpreadSheet appendRow: a.
     tmpCVSExporter setDateFormat: '"datum: "%d. %M. %y' language: nil.
     tmpCVSExporter setTimeFormat: '"cas: "%h:%m'.
     tmpCVSExporter export: tmpSpreadSheet.

     tmpFile:= FileStream new.
     tmpFile open: testFileName withMode: 'r'.

     self assert: ((tmpFile nextLine) = '100,200,300,400,500,600').
     self assert: ((tmpFile nextLine) = '700,800,"""ahoj""",100,110,120').
     self assert: ((tmpFile nextLine) = '130,140,150,160,170,').
     self assert: ((tmpFile nextLine) = '"""cas: ""11:54",7,"""datum: ""13. 1. 2008",,,').
     self assert: ((tmpFile nextLine) = nil).

     tmpFile close.
     testFileName asFilename delete.
! !

