'From Smalltalk/X, Version:5.2.8 on 16-01-2008 at 01:34:52'                     !

"{ Package: 'SpreadSheet:all' }"

SpreadSheetImporter subclass:#SpreadSheetImporterCSV
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

!SpreadSheetImporterCSV methodsFor:'private'!

importCell: fromStream
        "get one cell from open stream"

        | string |

        (fromStream peek = $,) ifTrue: [^ nil].   "nothing defined - empty cell"
        (fromStream peek = $")
                ifTrue:
                 [
                        "absorb leading quote"
                        fromStream next.
                        string := self importString: fromStream.
                        "absorb trailing quote"
                        (fromStream next = $") ifFalse: [self error: 'Broken CSV - expecting trailing quote'].
                        ^ string
                 ]
                ifFalse:
                 [
                        "accumulate possible number"
                        string := String new.
                        [fromStream atEnd or: [fromStream peek = Character lf 
                                          or: [fromStream peek = Character return
                                          or: [fromStream peek = $,
                        ]]]] whileFalse: [string := string, fromStream next].
                        ^ string asNumber
                 ].

        ^ nil
!

importRow: fromStream
        "get one row from open stream"

        | row cell |

        row := OrderedCollection new.

        [fromStream atEnd or: [fromStream peek = Character lf  or: [fromStream peek = Character return]]] whileFalse:
         [
                cell := self importCell: fromStream.
                row add: cell.
                (fromStream atEnd or: [fromStream peek = Character lf  or: [fromStream peek = Character return]]) ifFalse:
                        "absorb comma"
                        [(fromStream next = $,) ifFalse: [self error: 'Broken CSV - expecting separating comma']]
         ].

        ^ row asArray
!

importString: fromStream
        "get one string from open stream"
        "single quote means end of string (or EOF)"
        "double quote means quote (escaping)"

        | string |

        string := String new.

        [fromStream atEnd] whileFalse:
         [
                (fromStream peek = $")
                        ifFalse: [string := string, fromStream next]
                        ifTrue:
                         [
                                fromStream next.
                                "was it end or escaped quote?"
                                (fromStream peek = $")
                                        ifFalse: [fromStream backStep. ^ string]   "this grammar is not LL(1) and trailing quote is absorbed in caller"
                                        ifTrue: [string := string, fromStream next]
                         ].
         ].

        ^ string
!

importTable: fromStream
        "get whole table from open stream"

        | table row |

        table := SpreadSheet new.

        [fromStream atEnd] whileFalse:
         [
                row := self importRow: fromStream.
                table appendRow: row.
                (fromStream atEnd) ifFalse:
                 [
                        "absorb CR/LF - MAC, Win or Unix format"
                        (fromStream peek = Character return)
                                ifTrue: [fromStream next. (fromStream peek = Character lf) ifTrue:[fromStream next]]
                                ifFalse: [(fromStream next = Character lf) ifFalse: [self error: 'Broken CSV - expecting line feed']]
                 ].
         ].

        ^ table
! !

!SpreadSheetImporterCSV methodsFor:'reading'!

import
        "import data from file to new SpreadSheet"
        "import format: CSV (comma separated values)"

        | stream table |

        stream := FileStream readonlyFileNamed: filename.
        stream text.
        table := self importTable: stream.
        stream close.
        ^ table
! !

