'From Smalltalk/X, Version:5.2.8 on 16-01-2008 at 01:34:53'                     !

"{ Package: 'SpreadSheet:all' }"

TestCase subclass:#SpreadSheetImporterCSVTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

!SpreadSheetImporterCSVTests methodsFor:'test'!

testImporting01
        "importing testing table - fixed numbers"

        | stream table filename |

        filename := '~SpreadSheetTest'.
        stream := FileStream newFileForWritingNamed: filename.
        stream nextPutAll:
                '1,23', Character lf,
                '456,7890'.
        stream close.

        table := (SpreadSheetImporterCSV newWithFile: filename) import.
        filename asFilename delete.

        self assert: (table rowsCount = 2).
        self assert: (table columnsCount = 2).

        self assert: ((table cellAtRow: 1 column: 1) = 1).
        self assert: ((table cellAtRow: 1 column: 2) = 23).
        self assert: ((table cellAtRow: 2 column: 1) = 456).
        self assert: ((table cellAtRow: 2 column: 2) = 7890).
!

testImporting02
        "importing testing table - floating numbers"

        | stream table filename |

        filename := '~SpreadSheetTest'.
        stream := FileStream newFileForWritingNamed: filename.
        stream nextPutAll:
                '1,0.2', Character lf,
                '.3,4.5'.
        stream close.

        table := (SpreadSheetImporterCSV newWithFile: filename) import.
        filename asFilename delete.

        self assert: (table rowsCount = 2).
        self assert: (table columnsCount = 2).

        self assert: ((table cellAtRow: 1 column: 1) = 1).
        self assert: ((table cellAtRow: 1 column: 2) = 0.2).
        self assert: ((table cellAtRow: 2 column: 1) = 0.3).
        self assert: ((table cellAtRow: 2 column: 2) = 4.5).
!

testImporting03
        "importing testing table - strings"

        | stream table filename |

        filename := '~SpreadSheetTest'.
        stream := FileStream newFileForWritingNamed: filename.
        stream nextPutAll:
                '"0","a"', Character lf,
                '"bc"," d e "'.
        stream close.

        table := (SpreadSheetImporterCSV newWithFile: filename) import.
        filename asFilename delete.

        self assert: (table rowsCount = 2).
        self assert: (table columnsCount = 2).

        self assert: ((table cellAtRow: 1 column: 1) = '0').
        self assert: ((table cellAtRow: 1 column: 2) = 'a').
        self assert: ((table cellAtRow: 2 column: 1) = 'bc').
        self assert: ((table cellAtRow: 2 column: 2) = ' d e ').
!

testImporting04
        "importing testing table - quoting"

        | stream table filename |

        filename := '~SpreadSheetTest'.
        stream := FileStream newFileForWritingNamed: filename.
        stream nextPutAll:
                '"",""""', Character lf,
                '",",""","""'.
        stream close.

        table := (SpreadSheetImporterCSV newWithFile: filename) import.
        filename asFilename delete.

        self assert: (table rowsCount = 2).
        self assert: (table columnsCount = 2).

        self assert: ((table cellAtRow: 1 column: 1) = '').
        self assert: ((table cellAtRow: 1 column: 2) = '"').
        self assert: ((table cellAtRow: 2 column: 1) = ',').
        self assert: ((table cellAtRow: 2 column: 2) = '","').
!

testImporting05
        "importing testing table - linefeeds, empty lines"

        | stream table filename |

        filename := '~SpreadSheetTest'.
        stream := FileStream newFileForWritingNamed: filename.
        stream nextPutAll:
                '"UNX"', Character lf,
                         Character lf,
                '"WIN"', Character return,  Character lf,
                         Character return,  Character lf,
                '"MAC"', Character return,
                         Character return.
        stream close.

        table := (SpreadSheetImporterCSV newWithFile: filename) import.
        filename asFilename delete.

        self assert: (table rowsCount = 6).
        self assert: (table columnsCount = 1).

        self assert: ((table cellAtRow: 1 column: 1) = 'UNX').
        self assert: ((table cellAtRow: 2 column: 1) = nil).
        self assert: ((table cellAtRow: 3 column: 1) = 'WIN').
        self assert: ((table cellAtRow: 4 column: 1) = nil).
        self assert: ((table cellAtRow: 5 column: 1) = 'MAC').
        self assert: ((table cellAtRow: 6 column: 1) = nil).
!

testImporting06
        "importing testing table - mixed example"

        | stream table filename |

        filename := '~SpreadSheetTest'.
        stream := FileStream newFileForWritingNamed: filename.
        stream nextPutAll:
                '1,23,456', Character lf,
                '0.1,.2,1.2', Character lf,
                '"a","bc"," d e "', Character lf,
                ',"",""""'.
        stream close.

        table := (SpreadSheetImporterCSV newWithFile: filename) import.
        filename asFilename delete.

        self assert: (table rowsCount = 4).
        self assert: (table columnsCount = 3).

        self assert: ((table cellAtRow: 1 column: 1) = 1).
        self assert: ((table cellAtRow: 1 column: 2) = 23).
        self assert: ((table cellAtRow: 1 column: 3) = 456).
        self assert: ((table cellAtRow: 2 column: 1) = 0.1).
        self assert: ((table cellAtRow: 2 column: 2) = 0.2).
        self assert: ((table cellAtRow: 2 column: 3) = 1.2).
        self assert: ((table cellAtRow: 3 column: 1) = 'a').
        self assert: ((table cellAtRow: 3 column: 2) = 'bc').
        self assert: ((table cellAtRow: 3 column: 3) = ' d e ').
        self assert: ((table cellAtRow: 4 column: 1) = nil).
        self assert: ((table cellAtRow: 4 column: 2) = '').
        self assert: ((table cellAtRow: 4 column: 3) = '"').
! !

